from gui_automation.gui_automation import GuiAuto
from gui_automation.image_detection import TM_SQDIFF_NORMED, TM_CCOEFF_NORMED, TM_CCORR_NORMED
from gui_automation.image_loader import load_images
name = "gui_automation"
